"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const XPath = __importStar(require("xpath"));
const Constants_1 = require("./Constants");
const Utils_1 = require("./Utils");
function addCheckpoint(document, element, options) {
    let { count, level, nodeLimit } = {
        count: 0,
        level: 0,
        nodeLimit: Constants_1.DefaultNodeLimit,
        ...options
    };
    if (element.firstChild && element.firstChild.nodeType === Constants_1.NodeTypes.TEXT_NODE)
        element.replaceChild(document.createTextNode(`${element.firstChild.nodeValue || ""}${Constants_1.CheckpointPrefix}${count}${Constants_1.CheckpointSuffix}`), element.firstChild);
    else
        element.nodeValue = `${Constants_1.CheckpointPrefix}${count}${Constants_1.CheckpointSuffix}`;
    count++;
    if (count < nodeLimit)
        for (let index = 0; index < element.childNodes.length && count < nodeLimit; index++) {
            const node = element.childNodes.item(index);
            if (node.nodeType !== Constants_1.NodeTypes.ELEMENT_NODE)
                continue;
            count = addCheckpoint(document, node, { nodeLimit, count, level: level + 1 });
        }
    if (element.nextSibling && element.nextSibling.nodeType === Constants_1.NodeTypes.TEXT_NODE)
        element.parentNode.replaceChild(document.createTextNode(`${element.nextSibling.nodeValue || ""}${Constants_1.CheckpointPrefix}${count}${Constants_1.CheckpointSuffix}`), element.nextSibling);
    else if (element.parentNode)
        element.parentNode.appendChild(document.createTextNode(`${Constants_1.CheckpointPrefix}${count}${Constants_1.CheckpointSuffix}`));
    count++;
    return count;
}
exports.addCheckpoint = addCheckpoint;
;
function deleteQuotationTags(document, element, quotationCheckpoints, options) {
    let isTagInQuotation = true;
    let { count, level, nodeLimit, shouldPreserveTable } = {
        count: 0,
        level: 0,
        shouldPreserveTable: false,
        nodeLimit: Constants_1.DefaultNodeLimit,
        ...options
    };
    if (quotationCheckpoints[count] && !shouldPreserveTable) {
        if (element.firstChild && element.firstChild.nodeType === Constants_1.NodeTypes.TEXT_NODE)
            element.replaceChild(document.createTextNode(""), element.firstChild);
        else
            element.nodeValue = "";
    }
    else {
        isTagInQuotation = false;
    }
    count++;
    const shouldPreserveTableAnyway = shouldPreserveTable || (!isTagInQuotation && element.nodeName === 'table');
    const quotationChildren = new Array();
    if (count < nodeLimit)
        for (let index = 0; index < element.childNodes.length && count < nodeLimit; index++) {
            const node = element.childNodes.item(index);
            if (node.nodeType !== Constants_1.NodeTypes.ELEMENT_NODE)
                continue;
            let isChildTagInQuotation;
            ({ count, isTagInQuotation: isChildTagInQuotation } = deleteQuotationTags(document, node, quotationCheckpoints, {
                nodeLimit,
                count,
                level: level + 1,
                shouldPreserveTable: shouldPreserveTableAnyway
            }));
            if (!isChildTagInQuotation)
                continue;
            quotationChildren.push(node);
        }
    if (quotationCheckpoints[count] && !shouldPreserveTableAnyway) {
        if (element.nextSibling && element.nextSibling.nodeType === Constants_1.NodeTypes.TEXT_NODE)
            element.parentNode.replaceChild(document.createTextNode(""), element.nextSibling);
    }
    else {
        isTagInQuotation = false;
    }
    count++;
    if (!isTagInQuotation && !shouldPreserveTableAnyway)
        for (const node of quotationChildren)
            node.parentNode.removeChild(node);
    return {
        count,
        isTagInQuotation
    };
}
exports.deleteQuotationTags = deleteQuotationTags;
function shouldNotRemoveQuote(node, options) {
    return Boolean(options && options.onlyRemoveEmptyBlocks && node.textContent.trim() !== '');
}
function cutGmailQuote(document, options) {
    const gmailQuote = XPath.select("//*[contains(@class, 'gmail_quote')]", document, true);
    if (!gmailQuote || (gmailQuote.textContent && Utils_1.isStartOfForwardedMessage(gmailQuote.textContent)) || shouldNotRemoveQuote(gmailQuote, options))
        return false;
    gmailQuote.parentNode.removeChild(gmailQuote);
    return true;
}
exports.cutGmailQuote = cutGmailQuote;
;
function cutMicrosoftQuote(document, options) {
    let splitter = XPath.select("//*[local-name(.)='div' and @style='border:none;" +
        "border-top:solid #B5C4DF 1.0pt;" +
        "padding:3.0pt 0cm 0cm 0cm']|" +
        "//*[local-name(.)='div' and @style='padding-top: 5px; " +
        "border-top-color: rgb(229, 229, 229); " +
        "border-top-width: 1px; border-top-style: solid;']", document, true);
    if (splitter) {
        if (splitter.parentElement && splitter === splitter.parentElement.children[0])
            splitter = splitter.parentNode;
    }
    else {
        splitter = XPath.select("//*[local-name(.)='div']" +
            "/*[local-name(.)='div' and @class='MsoNormal' and @align='center' and @style='text-align:center']" +
            "/*[local-name(.)='font']" +
            "/*[local-name(.)='span']" +
            "/*[local-name(.)='hr' and @size='3' and @width='100%' and @align='center' and @tabindex='-1']", document, true);
        if (splitter) {
            if (splitter.parentNode)
                splitter = splitter.parentNode;
            if (splitter.parentNode)
                splitter = splitter.parentNode;
            if (splitter.parentNode)
                splitter = splitter.parentNode;
            if (splitter.parentNode)
                splitter = splitter.parentNode;
        }
    }
    if (!splitter || shouldNotRemoveQuote(splitter, options))
        return false;
    while (splitter.nextSibling)
        splitter.parentNode.removeChild(splitter.nextSibling);
    splitter.parentNode.removeChild(splitter);
    return true;
}
exports.cutMicrosoftQuote = cutMicrosoftQuote;
;
function cutZimbraQuote(document, options) {
    const splitter = XPath.select("//*[local-name(.)='hr' and @data-marker=\"__DIVIDER__\"]", document, true);
    if (!splitter || shouldNotRemoveQuote(splitter, options))
        return false;
    splitter.parentNode.removeChild(splitter);
    return true;
}
exports.cutZimbraQuote = cutZimbraQuote;
;
function cutById(document, options) {
    let found = false;
    for (const quoteId of Constants_1.QuoteIds) {
        const quote = XPath.select(`//*[@id="${quoteId}"]`, document, true);
        if (!quote || shouldNotRemoveQuote(quote, options))
            continue;
        found = true;
        quote.parentNode.removeChild(quote);
    }
    return found;
}
exports.cutById = cutById;
;
function cutBlockquote(document, options) {
    const quote = XPath.select("(.//*[local-name(.)='blockquote'])" +
        "[not(@class=\"gmail_quote\") and not(ancestor::blockquote)]" +
        "[last()]", document, true);
    if (!quote || shouldNotRemoveQuote(quote, options))
        return false;
    quote.parentNode.removeChild(quote);
    return true;
}
exports.cutBlockquote = cutBlockquote;
;
//# sourceMappingURL=HtmlQuotations.js.map