"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const XPath = __importStar(require("xpath"));
const Constants_1 = require("./Constants");
const Regexp_1 = require("./Regexp");
function findDelimiter(messageBody) {
    var match = Regexp_1.DelimiterRegexp.exec(messageBody);
    return match ? match[0] : "\n";
}
exports.findDelimiter = findDelimiter;
;
function splitLines(str) {
    return str.split(/\r?\n/);
}
exports.splitLines = splitLines;
;
function isStartOfForwardedMessage(str) {
    return Boolean(matchStart(str.trim(), Regexp_1.ForwardRegexp));
}
exports.isStartOfForwardedMessage = isStartOfForwardedMessage;
;
function matchStart(str, regexp) {
    const startRegexp = new RegExp(`^${regexp.source}`, regexp.flags);
    let match = str.match(startRegexp);
    return !match || match.index > 0 ? null : match;
}
exports.matchStart = matchStart;
;
function elementToText(element, { ignoreBlockTags } = {}) {
    const styleNodes = XPath.select("//style", element);
    for (const styleNode of styleNodes)
        styleNode.parentNode.removeChild(styleNode);
    const commentNodes = XPath.select("//comment()", element);
    for (const commentNode of commentNodes)
        commentNode.parentNode.removeChild(commentNode);
    let text = "";
    const allNodes = XPath.select("//*", element);
    for (const node of allNodes) {
        const nodeText = extractTextFromNode(node, text);
        if (nodeText.length > 1) {
            const nodeName = node.nodeName.toLowerCase();
            if (!ignoreBlockTags && Constants_1.BlockTags.includes(nodeName))
                text += "\n";
            if (node.nodeName.toLowerCase() === "li")
                text += "  * ";
            text += `${nodeText} `;
            const href = node.attributes.getNamedItem("href");
            if (href)
                text += `(${href}) `;
            const src = node.attributes.getNamedItem("src");
            if (src)
                text += `(${src}) `;
        }
    }
    return removeExcessiveNewlines(text);
}
exports.elementToText = elementToText;
;
function extractTextFromNode(node, text) {
    let nodeValue = (node.nodeValue || (node.firstChild && node.firstChild.nodeType === Constants_1.NodeTypes.TEXT_NODE && node.firstChild.nodeValue) || '').trim();
    const siblingValue = ((node.nextSibling && node.nextSibling.nodeType === Constants_1.NodeTypes.TEXT_NODE && node.nextSibling.nodeValue) || '').trim();
    if (Constants_1.HardbreakTags.indexOf(node.nodeName.toLowerCase()) >= 0
        && text && text[text.length - 1] !== "\n")
        nodeValue += "\n";
    let nodeText = nodeValue + siblingValue;
    return nodeText.replace('\\n', '\n');
}
function normalizeHtmlDocument(document) {
    const matchStart = document.match(/^(\s*<html[^>]*>)?(\s*<head[^>]*>.*<\/head>)?(\s*<body[^>]*>)?/im);
    const matchEnd = document.match(/(<\/body>\s*)?(<\/html>\s*)?$/im);
    if (!matchStart[3])
        document = matchStart[0].trim() +
            "<body>" +
            document.slice(matchStart[0].length);
    if (!matchStart[1])
        document = "<html>" + document;
    if (!matchEnd[1])
        document = (matchEnd[0] ? document.slice(0, -matchEnd[0].length) : document) +
            "</body>" +
            matchEnd[0].trim();
    if (!matchEnd[2])
        document = document + "</html>";
    return document;
}
exports.normalizeHtmlDocument = normalizeHtmlDocument;
;
function removeExcessiveNewlines(src) {
    return src.replace(/\n{2,10}/g, "\n\n").trim();
}
//# sourceMappingURL=Utils.js.map